import time
import spidev
import gpio as GPIO
import sys
import matplotlib.pyplot as plt

PIN_CHARGE_ENABLE=4
GPIO.setup(PIN_CHARGE_ENABLE, GPIO.OUT)
GPIO.output(PIN_CHARGE_ENABLE, GPIO.LOW)

# Capacitor Terminal Voltage
VTERM=2.7
# VTERM=3.0
# VTERM=3.6
# VTERM=5.0

# Measurement Voltage levels according to CDE guide
# V1 = VTERM * 0.8
# V2 = VTERM * 0.4
V1 = 2.5
V2 = 2.0

# ADC Reference (VCC)
VCC=3.315

# Charge state delay (sec) - delay between charging and discharging phase (voltage is maintained in this stage)
DELAY = 5 * 60

# Discharge current
DISCHARGE_CURRENT=0.01895
# DISCHARGE_CURRENT=0.00359

max11410_spi = spidev.SpiDev()
max11410_spi.open(0, 0)
max11410_spi.max_speed_hz = 1000000
max11410_spi.mode = 0

def max11410_write_reg8(addr, val):
	max11410_spi.xfer([addr, val])

def max11410_write_reg16(addr, val):
	max11410_spi.xfer([addr, val >> 8, val & 0xFF])

def max11410_read_reg24(addr):
	received_data = max11410_spi.xfer([addr | 0x80, 0, 0, 0])
	return received_data[3] | (received_data[2] << 8) | (received_data[1] << 16)

def max11410_read_reg24_bytes(addr):
	received_data = max11410_spi.xfer([addr | 0x80, 0, 0, 0])
	return [received_data[3] ,received_data[2], received_data[1]]

def max11410_wait_for_conversion_complete():
	while True:
		if max11410_read_reg24(0x38) & 0x10:
			return
		
def max11410_measure_voltage():
	max11410_write_reg8(0x01, 0x00) # start single conversion
	max11410_wait_for_conversion_complete()
	value = int.from_bytes(max11410_read_reg24_bytes(0x30), "little", signed="True")
	return (VCC * value / 2**23) * 5.336 / 2.470

# reset sensor
max11410_write_reg8(0x00, 0x03)

#configure sensor
max11410_write_reg8(0x08, 0x3F) # select filter:    FILTER=SINC4 RATE=1920sps
max11410_write_reg8(0x09, 0x03) # select reference: P=AVCC N=AGND
max11410_write_reg8(0x0B, 0x9A) # select input:     P=AIN9 N=AGND

print("Charging")
GPIO.output(PIN_CHARGE_ENABLE, GPIO.HIGH)

while True:
	actual = max11410_measure_voltage()
	print(str("Actual voltage: {:.5f}".format(actual)) + "        ", end="\r")

	if (actual > VTERM):
		GPIO.output(PIN_CHARGE_ENABLE, GPIO.LOW)
		print("Terminal voltage violated! Stopping charge. Adjust voltage supply before trying again!")
		sys.exit(1)
		break

	if (actual > VTERM - 0.2):
		break


print("Charging complete. Remaining charge for " + str(DELAY) + " sec.", end="\r")
print()

delay_end = time.time() + DELAY
while time.time() < delay_end:
	actual = max11410_measure_voltage()
	print(" " + str(round(delay_end - time.time())) + str(" Actual voltage: {:.5f}".format(actual)) + "        ", end="\r")

	if (actual > VTERM):
		GPIO.output(PIN_CHARGE_ENABLE, GPIO.LOW)
		print("Terminal voltage violated! Stopping charge. Adjust voltage supply before trying again!")
		sys.exit(1)
		break

print("Discharging.                                      ", end="\r")
print()
GPIO.output(PIN_CHARGE_ENABLE, GPIO.LOW)

v1_hit = False
v2_hit = False
while True:
	actual = max11410_measure_voltage()
	print(str("Actual voltage: {:.5f}".format(actual)) + "        ", end="\r")

	if (actual > VTERM):
		GPIO.output(PIN_CHARGE_ENABLE, GPIO.LOW)
		print("Terminal voltage violated! Stopping charge. Adjust voltage supply before trying again!")
		sys.exit(1)
		break

	if (actual < V1 and not v1_hit):
		v1_hit = True
		v1_hit_time = time.time()
		
	if (actual < V2 and not v2_hit):
		v2_hit = True
		v2_hit_time = time.time()
		break

capacitance = DISCHARGE_CURRENT * (v2_hit_time - v1_hit_time) / (V1 - V2)
print("Capacitance =", capacitance)
